/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: carol@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * --------------------------------------------------------------------------
 * $Id: SlaveServerClusterViewManager.java 1892 2008-07-07 12:54:22Z loris $
 * --------------------------------------------------------------------------
 */

package org.ow2.cmi.controller.server.impl.coordinator.slave;

import java.util.Hashtable;

import javax.jms.JMSException;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import net.jcip.annotations.ThreadSafe;

import org.ow2.cmi.controller.server.DistributedObjectInfo;
import org.ow2.cmi.controller.server.impl.coordinator.CoordinatedServerClusterViewManager;
import org.ow2.cmi.reference.CMIReference;
import org.ow2.cmi.reference.ServerId;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * @author Loris Bouzonnet
 */
@ThreadSafe
public class SlaveServerClusterViewManager extends CoordinatedServerClusterViewManager {

    /**
     * Logger.
     */
    private static final Log logger = LogFactory
            .getLog(SlaveServerClusterViewManager.class);

    private TopicPublisher publisher;

    @Override
    protected void initServerConfig(final String contextFactory, final ServerId serverId, final Context cmiContext) {
        TopicConnection tc = getTopicConnection();
        try {
            if(tc == null) {
                Hashtable<String, String> env = new Hashtable<String, String>();
                env.put(Context.INITIAL_CONTEXT_FACTORY, contextFactory);
                env.put(Context.PROVIDER_URL, serverId.getProviderURL());
                Context context = new InitialContext(env);
                TopicConnectionFactory tcf = (TopicConnectionFactory) context.lookup("JTCF");
                context.close();
                tc = tcf.createTopicConnection("slave", "cmi");
                setTopicConnection(tc);
                TopicSession topicSession = tc.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
                setSession(topicSession);
                Topic topic = (Topic) context.lookup("cmiCoordinator");
                setTopic(topic);
                publisher = topicSession.createPublisher(topic);
                tc.start();
            }
        } catch (NamingException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (JMSException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        super.initServerConfig(contextFactory, serverId, cmiContext);
    }

    @Override
    public void doStart() {
        // TODO Auto-generated method stub

    }

    @Override
    public void doStop() {
        // TODO Auto-generated method stub

    }

    @Override
    protected void addDistributedObjectInfo(final String objectName,
            final DistributedObjectInfo distributedObjectInfo) {
        try {
            ObjectMessage msg = getSession().createObjectMessage(distributedObjectInfo);
            msg.setBooleanProperty("add", true);
            publisher.send(msg);
        } catch (JMSException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * Sets informations on the clustered object with the given name.
     *
     * @param objectName
     *            a name of object
     * @param distributedObjectInfo
     *            informations on the clustered object
     */
    @Override
    protected void setDistributedObjectInfo(final String objectName,
            final DistributedObjectInfo distributedObjectInfo) {
        try {
            ObjectMessage msg = getSession().createObjectMessage(distributedObjectInfo);
            msg.setBooleanProperty("add", false);
            publisher.send(msg);
        } catch (JMSException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @Override
    public void addCMIReference(final CMIReference cmiReference) {
        try {
            ObjectMessage msg = getSession().createObjectMessage(cmiReference);
            msg.setBooleanProperty("add", true);
            publisher.send(msg);
        } catch (JMSException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @Override
    public void removeCMIReference(final CMIReference cmiReference) {
        try {
            ObjectMessage msg = getSession().createObjectMessage(cmiReference);
            msg.setBooleanProperty("add", false);
            publisher.send(msg);
        } catch (JMSException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

}
